==> fluffybox-0.1.3.tar.gz: lib/src/sqflite.dart <==
import 'dart:convert';
import 'dart:typed_data';

import 'package:sqflite_common/sqlite_api.dart';

class BoxCollection {
  final Database _db;
  final Set<String> boxNames;

  BoxCollection(this._db, this.boxNames);

  static Future<BoxCollection> open(
    String name,
    Set<String> boxNames, {
    int version = 1,
    dynamic sqfliteDatabase,
  }) async {
    if (sqfliteDatabase is! Database) {
      throw ('You must provide a Database `sqfliteDatabase` for FluffyBox on native.');
    }
    final batch = sqfliteDatabase.batch();
    batch.execute('PRAGMA page_size = 8192');
    batch.execute('PRAGMA cache_size = 16384');
    batch.execute('PRAGMA temp_store = MEMORY');
    batch.rawQuery('PRAGMA journal_mode = WAL');
    for (final name in boxNames) {
      batch.execute(
        'CREATE TABLE IF NOT EXISTS $name (k TEXT PRIMARY KEY, v BLOB)',
      );
    }
    await batch.commit(noResult: true);
    return BoxCollection(sqfliteDatabase, boxNames);
  }

  Box<V> openBox<V>(String name) {
    if (!boxNames.contains(name)) {
      throw ('Box with name $name is not in the known box names of this collection.');
    }
    return Box<V>(name, this);
  }

  List<Future<void> Function(Batch txn)>? _txnCache;

  Future<void> transaction(
    Future<void> Function() action, {
    List<String>? boxNames,
    bool readOnly = false,
  }) async {
    boxNames ??= this.boxNames.toList();
    _txnCache = [];
    await action();
    final cache = List<Future<void> Function(Batch txn)>.from(_txnCache!);
    _txnCache = null;
    if (cache.isEmpty) return;
    await _db.transaction((txn) async {
      final batch = txn.batch();
      for (final fun in cache) {
        await fun(batch);
      }
      await batch.commit(noResult: true);
    });
    return;
  }

  void close() {
    //_db.close();
  }
}

class Box<V> {
  final String name;
  final BoxCollection boxCollection;
  final Map<String, V?> _cache = {};
  Set<String>? _cachedKeys;
  bool get _keysCached => _cachedKeys != null;

  Box(this.name, this.boxCollection) {
    if (!(V is String ||
        V is bool ||
        V is int ||
        V is Object ||
        V is List<Object?> ||
        V is Map<String, Object?> ||
        V is double)) {
      throw ('Value type ${V.runtimeType} is not one of the allowed value types {String, int, double, List<Object?>, Map<String, Object?>}.');
    }
  }

  V? _fromBlob(Object? value) {
    if (value == null) return null;
    if (value is! Uint8List) {
      throw ('Corrupted value field in FluffyBox. Expected Uint8List but got type ${value.runtimeType}: $value');
    }
    var str = utf8.decode(value);
    switch (V.toString()) {
      case 'int':
        return int.parse(str) as V;
      case 'double':
        return double.parse(str) as V;
      case 'bool':
        return (str == 'true') as V;
      case 'List<dynamic>':
      case 'Map<dynamic, dynamic>':
        return jsonDecode(str) as V;
      case 'String':
      default:
        return str.substring(1, str.length - 1) as V;
    }
  }

  Future<List<String>> getAllKeys([Transaction? txn]) async {
    if (_keysCached) return _cachedKeys!.toList();

    final query = 'SELECT k FROM $name';
    final result =
        await (txn?.rawQuery(query) ?? boxCollection._db.rawQuery(query));
    final keys = result.map((row) => row['k'] as String).toList();

    _cachedKeys = keys.toSet();
    return keys;
  }

  Future<Map<String, V>> getAllValues([Transaction? txn]) async {
    final query = 'SELECT * FROM $name';
    final result =
        await (txn?.rawQuery(query) ?? boxCollection._db.rawQuery(query));
    return Map.fromEntries(result
        .map((row) => MapEntry(row['k'] as String, _fromBlob(row['v'])!)));
  }

  Future<V?> get(String key, [Transaction? txn]) async {
    if (_cache.containsKey(key)) return _cache[key];

    final query = 'SELECT v FROM $name WHERE k = ?';
    final params = [key];
    final result = await (txn?.rawQuery(query, params) ??
        boxCollection._db.rawQuery(query, params));

    final value = result.isEmpty ? null : _fromBlob(result.single['v']);
    _cache[key] = value;
    return value;
  }

  Future<List<V?>> getAll(List<String> keys, [Transaction? txn]) async {
    if (!keys.any((key) => !_cache.containsKey(key))) {
      return keys.map((key) => _cache[key]).toList();
    }

    late final List<V?> list;
    if (txn == null) {
      await boxCollection._db.transaction((txn) async {
        list = await _queryAllInTransaction(keys, txn);
      });
    } else {
      list = await _queryAllInTransaction(keys, txn);
    }

    for (var i = 0; i < keys.length; i++) {
      _cache[keys[i]] = list[i];
    }
    return list;
  }

  Future<List<V?>> _queryAllInTransaction(
      List<String> keys, Transaction txn) async {
    final list = <V?>[];
    final query = 'SELECT * FROM $name WHERE k=?';
    for (var i = 0; i < keys.length; i++) {
      final params = [keys[i]];
      final result = await txn.rawQuery(query, params);
      final value = result.isEmpty ? null : _fromBlob(result.single['v']);
      list.add(value);
    }
    return list;
  }

  Future<void> put(String key, V val, [Batch? txn]) async {
    if (boxCollection._txnCache != null) {
      boxCollection._txnCache!.add((txn) => put(key, val, txn));
      _cache[key] = val;
      _cachedKeys?.add(key);
      return;
    }

    final query = 'INSERT OR REPLACE INTO $name (k,v) VALUES(?,?)';
    final params = [key, Uint8List.fromList(utf8.encode(jsonEncode(val)))];
    if (txn == null) {
      await boxCollection._db.rawInsert(query, params);
    } else {
      txn.rawInsert(query, params);
    }

    _cache[key] = val;
    _cachedKeys?.add(key);
    return;
  }

  Future<void> delete(String key, [Batch? txn]) async {
    if (boxCollection._txnCache != null) {
      boxCollection._txnCache!.add((txn) => delete(key, txn));
      _cache.remove(key);
      _cachedKeys?.remove(key);
      return;
    }

    final query = 'DELETE FROM $name WHERE k=?';
    final params = [key];
    if (txn == null) {
      await boxCollection._db.rawDelete(query, params);
    } else {
      txn.rawDelete(query, params);
    }

    _cache.remove(key);
    _cachedKeys?.remove(key);
    return;
  }

  Future<void> deleteAll(List<String> keys, [Batch? txn]) async {
    if (boxCollection._txnCache != null) {
      boxCollection._txnCache!.add((txn) => deleteAll(keys, txn));
      keys.forEach(_cache.remove);
      _cachedKeys?.removeAll(keys);
      return;
    }

    final placeholder = keys.map((_) => '?').join(',');
    final query = 'DELETE FROM $name WHERE k IN ($placeholder)';
    final params = [...keys];
    if (txn == null) {
      await boxCollection._db.rawDelete(query, params);
    } else {
      txn.rawDelete(query, params);
    }

    for (final key in keys) {
      _cache.remove(key);
      _cachedKeys?.removeAll(keys);
    }
    return;
  }

  Future<void> clear([Batch? txn]) async {
    if (boxCollection._txnCache != null) {
      boxCollection._txnCache!.add((txn) => clear(txn));
      _cache.clear();
      _cachedKeys = null;
      return;
    }

    final query = 'DELETE FROM $name';
    if (txn == null) {
      await boxCollection._db.rawDelete(query);
    } else {
      txn.rawDelete(query);
    }

    _cache.clear();
    _cachedKeys = null;
    return;
  }
}

==> fluffybox-0.1.4.tar.gz: lib/src/sqflite.dart <==
import 'dart:convert';
import 'dart:typed_data';

import 'package:sqflite_common/sqlite_api.dart';

class BoxCollection {
  final Database _db;
  final Set<String> boxNames;

  BoxCollection(this._db, this.boxNames);

  static Future<BoxCollection> open(
    String name,
    Set<String> boxNames, {
    int version = 1,
    dynamic sqfliteDatabase,
  }) async {
    if (sqfliteDatabase is! Database) {
      throw ('You must provide a Database `sqfliteDatabase` for FluffyBox on native.');
    }
    await sqfliteDatabase.execute('PRAGMA page_size = 8192');
    await sqfliteDatabase.execute('PRAGMA cache_size = 16384');
    await sqfliteDatabase.execute('PRAGMA temp_store = MEMORY');
    await sqfliteDatabase.rawQuery('PRAGMA journal_mode = WAL');
    final batch = sqfliteDatabase.batch();
    for (final name in boxNames) {
      batch.execute(
        'CREATE TABLE IF NOT EXISTS $name (k TEXT PRIMARY KEY, v BLOB)',
      );
    }
    await batch.commit(noResult: true);
    return BoxCollection(sqfliteDatabase, boxNames);
  }

  Box<V> openBox<V>(String name) {
    if (!boxNames.contains(name)) {
      throw ('Box with name $name is not in the known box names of this collection.');
    }
    return Box<V>(name, this);
  }

  List<Future<void> Function(Batch txn)>? _txnCache;

  Future<void> transaction(
    Future<void> Function() action, {
    List<String>? boxNames,
    bool readOnly = false,
  }) async {
    boxNames ??= this.boxNames.toList();
    _txnCache = [];
    await action();
    final cache = List<Future<void> Function(Batch txn)>.from(_txnCache!);
    _txnCache = null;
    if (cache.isEmpty) return;
    await _db.transaction((txn) async {
      final batch = txn.batch();
      for (final fun in cache) {
        await fun(batch);
      }
      await batch.commit(noResult: true);
    });
    return;
  }

  void close() {
    //_db.close();
  }
}

class Box<V> {
  final String name;
  final BoxCollection boxCollection;
  final Map<String, V?> _cache = {};
  Set<String>? _cachedKeys;
  bool get _keysCached => _cachedKeys != null;

  Box(this.name, this.boxCollection) {
    if (!(V is String ||
        V is bool ||
        V is int ||
        V is Object ||
        V is List<Object?> ||
        V is Map<String, Object?> ||
        V is double)) {
      throw ('Value type ${V.runtimeType} is not one of the allowed value types {String, int, double, List<Object?>, Map<String, Object?>}.');
    }
  }

  V? _fromBlob(Object? value) {
    if (value == null) return null;
    if (value is! Uint8List) {
      throw ('Corrupted value field in FluffyBox. Expected Uint8List but got type ${value.runtimeType}: $value');
    }
    var str = utf8.decode(value);
    switch (V.toString()) {
      case 'int':
        return int.parse(str) as V;
      case 'double':
        return double.parse(str) as V;
      case 'bool':
        return (str == 'true') as V;
      case 'List<dynamic>':
      case 'Map<dynamic, dynamic>':
        return jsonDecode(str) as V;
      case 'String':
      default:
        return str.substring(1, str.length - 1) as V;
    }
  }

  Future<List<String>> getAllKeys([Transaction? txn]) async {
    if (_keysCached) return _cachedKeys!.toList();

    final query = 'SELECT k FROM $name';
    final result =
        await (txn?.rawQuery(query) ?? boxCollection._db.rawQuery(query));
    final keys = result.map((row) => row['k'] as String).toList();

    _cachedKeys = keys.toSet();
    return keys;
  }

  Future<Map<String, V>> getAllValues([Transaction? txn]) async {
    final query = 'SELECT * FROM $name';
    final result =
        await (txn?.rawQuery(query) ?? boxCollection._db.rawQuery(query));
    return Map.fromEntries(result
        .map((row) => MapEntry(row['k'] as String, _fromBlob(row['v'])!)));
  }

  Future<V?> get(String key, [Transaction? txn]) async {
    if (_cache.containsKey(key)) return _cache[key];

    final query = 'SELECT v FROM $name WHERE k = ?';
    final params = [key];
    final result = await (txn?.rawQuery(query, params) ??
        boxCollection._db.rawQuery(query, params));

    final value = result.isEmpty ? null : _fromBlob(result.single['v']);
    _cache[key] = value;
    return value;
  }

  Future<List<V?>> getAll(List<String> keys, [Transaction? txn]) async {
    if (!keys.any((key) => !_cache.containsKey(key))) {
      return keys.map((key) => _cache[key]).toList();
    }

    late final List<V?> list;
    if (txn == null) {
      await boxCollection._db.transaction((txn) async {
        list = await _queryAllInTransaction(keys, txn);
      });
    } else {
      list = await _queryAllInTransaction(keys, txn);
    }

    for (var i = 0; i < keys.length; i++) {
      _cache[keys[i]] = list[i];
    }
    return list;
  }

  Future<List<V?>> _queryAllInTransaction(
      List<String> keys, Transaction txn) async {
    final list = <V?>[];
    final query = 'SELECT * FROM $name WHERE k=?';
    for (var i = 0; i < keys.length; i++) {
      final params = [keys[i]];
      final result = await txn.rawQuery(query, params);
      final value = result.isEmpty ? null : _fromBlob(result.single['v']);
      list.add(value);
    }
    return list;
  }

  Future<void> put(String key, V val, [Batch? txn]) async {
    if (boxCollection._txnCache != null) {
      boxCollection._txnCache!.add((txn) => put(key, val, txn));
      _cache[key] = val;
      _cachedKeys?.add(key);
      return;
    }

    final query = 'INSERT OR REPLACE INTO $name (k,v) VALUES(?,?)';
    final params = [key, Uint8List.fromList(utf8.encode(jsonEncode(val)))];
    if (txn == null) {
      await boxCollection._db.rawInsert(query, params);
    } else {
      txn.rawInsert(query, params);
    }

    _cache[key] = val;
    _cachedKeys?.add(key);
    return;
  }

  Future<void> delete(String key, [Batch? txn]) async {
    if (boxCollection._txnCache != null) {
      boxCollection._txnCache!.add((txn) => delete(key, txn));
      _cache.remove(key);
      _cachedKeys?.remove(key);
      return;
    }

    final query = 'DELETE FROM $name WHERE k=?';
    final params = [key];
    if (txn == null) {
      await boxCollection._db.rawDelete(query, params);
    } else {
      txn.rawDelete(query, params);
    }

    _cache.remove(key);
    _cachedKeys?.remove(key);
    return;
  }

  Future<void> deleteAll(List<String> keys, [Batch? txn]) async {
    if (boxCollection._txnCache != null) {
      boxCollection._txnCache!.add((txn) => deleteAll(keys, txn));
      keys.forEach(_cache.remove);
      _cachedKeys?.removeAll(keys);
      return;
    }

    final placeholder = keys.map((_) => '?').join(',');
    final query = 'DELETE FROM $name WHERE k IN ($placeholder)';
    final params = [...keys];
    if (txn == null) {
      await boxCollection._db.rawDelete(query, params);
    } else {
      txn.rawDelete(query, params);
    }

    for (final key in keys) {
      _cache.remove(key);
      _cachedKeys?.removeAll(keys);
    }
    return;
  }

  Future<void> clear([Batch? txn]) async {
    if (boxCollection._txnCache != null) {
      boxCollection._txnCache!.add((txn) => clear(txn));
      _cache.clear();
      _cachedKeys = null;
      return;
    }

    final query = 'DELETE FROM $name';
    if (txn == null) {
      await boxCollection._db.rawDelete(query);
    } else {
      txn.rawDelete(query);
    }

    _cache.clear();
    _cachedKeys = null;
    return;
  }
}
